@@ -1,4 +1,5 @@
-/*! A dynamically-allocated, fixed-size, buffer containing a `BitSlice` region.
+/*! A dynamically-allocated, fixed-size, buffer containing a [`BitSlice`]
+region.
 
 You can read the standard library’s [`alloc::boxed` module documentation][std]
 here.
@@ -6,8 +7,8 @@ here.
 This module defines the [`BitBox`] buffer, and all of its associated support
 code.
 
-`BitBox` is equivalent to `Box<[bool]>`, in its operation and in its
-relationship to the `BitSlice` and [`BitVec`] types. Most of the interesting
+[`BitBox`] is equivalent to `Box<[bool]>`, in its operation and in its
+relationship to the [`BitSlice`] and [`BitVec`] types. Most of the interesting
 work to be done on a bit-sequence is implemented in `BitSlice`, to which
 `BitBox` dereferences, and the box container itself only exists to maintain
 wonership and provide some specializations that cannot safely be done on
@@ -19,11 +20,11 @@ it when you have a bit-sequence whose width is either unknowable at compile-time
 or inexpressable in `BitArray`, and are constructing the sequence in a `BitVec`
 before freezing it.
 
-[`BitArray`]: ../array/struct.BitArray.html
-[`BitBox`]: struct.BitBox.html
-[`BitSlice`]: ../slice/struct.BitSlice.html
-[`BitVec`]: ../vec/struct.BitVec.html
-[std]: https://doc.rust-lang.org/alloc/boxed
+[`BitArray`]: crate::array::BitArray
+[`BitBox`]: crate::boxed::BitBox
+[`BitSlice`]: crate::slice::BitSlice
+[`BitVec`]: crate::vec::BitVec
+[std]: alloc::boxed
 !*/
 
 #![cfg(feature = "alloc")]
@@ -31,20 +32,24 @@ before freezing it.
 use crate::{
 	index::BitIdx,
 	mem::BitMemory,
+	mutability::Mut,
 	order::{
 		BitOrder,
 		Lsb0,
 	},
-	pointer::BitPtr,
+	ptr::{
+		BitPtr,
+		BitSpan,
+	},
 	slice::BitSlice,
 	store::BitStore,
+	vec::BitVec,
 };
 
 use alloc::boxed::Box;
 
 use core::{
 	mem::ManuallyDrop,
-	ptr::NonNull,
 	slice,
 };
 
@@ -53,15 +58,9 @@ use tap::pipe::Pipe;
 /** A frozen heap-allocated buffer of individual bits.
 
 This is essentially a [`BitVec`] that has frozen its allocation, and given up
-the ability to change size. It is analagous to `Box<[bool]>`, and is written to
-be as close as possible to drop-in replacable for it. This type contains almost
-no interesting behavior in its own right; it dereferences to [`BitSlice`] to
-manipulate its contents, and it converts to and from `BitVec` for allocation
-control.
-
-If you know the length of your bit sequence at compile-time, and it is
-expressible within the limits of [`BitArray`], you should prefer that type
-instead. Large `BitArray`s can be `Box`ed normally as desired.
+the ability to change size. It is analagous to `Box<[bool]>`. You should prefer
+[`BitArray`] over `BitBox` where possible, and may freely box it if you need the
+indirection.
 
 # Documentation
 
@@ -74,7 +73,7 @@ These sections look like this:
 
 # Original
 
-[`Box<[T]>`](https://doc.rust-lang.org/alloc/boxed/struct.Box.html)
+[`Box<[T]>`](alloc::boxed::Box)
 
 # API Differences
 
@@ -86,43 +85,42 @@ implement the full API surface of `Box<[bool]>`.
 
 Because `BitBox` is a fully-owned buffer, it is able to operate on its memory
 without concern for any other views that may alias. This enables it to
-specialize some `BitSlice` behavior to be faster or more efficient.
+specialize some [`BitSlice`] behavior to be faster or more efficient.
 
 # Type Parameters
 
-This takes the same two type parameters, `O: BitOrder` and `T: BitStore`, as
-`BitSlice`.
+This takes the same [`BitOrder`] and [`BitStore`] parameters as [`BitSlice`].
+Unlike `BitSlice`, it is restricted to only accept the fundamental integers as
+its `BitStore` arguments; `BitBox` buffers can never be aliased by other
+`BitBox`es, and do not need to share memory access.
 
 # Safety
 
-Like `BitSlice`, `BitBox` is exactly equal in size to `Box<[bool]>`, and is also
-absolutely representation-incompatible with it. You must never attempt to
-type-cast between `Box<[bool]>` and `BitBox` in any way, nor attempt to modify
-the memory value of a `BitBox` handle. Doing so will cause allocator and memory
-errors in your program, likely inducing a panic.
+`BitBox` is a wrapper over a `NonNull<BitSlice<O, T>>` pointer; this allows it
+to remain exactly two words in size, and means that it is subject to the same
+representational incompatibility restrictions as [`BitSlice`] references. You
+must never attempt to type-cast between `Box<[bool]>` and `BitBox` in any way,
+nor may you attempt to modify the memory value of a `BitBox` handle. Doing so
+will cause allocator and memory errors in your program, likely inducing a panic.
 
 Everything in the `BitBox` public API, even the `unsafe` parts, are guaranteed
-to have no more unsafety than their equivalent items in the standard library.
-All `unsafe` APIs will have documentation explicitly detailing what the API
-requires you to uphold in order for it to function safely and correctly. All
-safe APIs will do so themselves.
-
-# Performance
-
-Iteration over the buffer is governed by the `BitSlice` characteristics on the
-type parameter. You are generally better off using larger types when your buffer
-is a data collection rather than a specific I/O protocol buffer.
+to have no more unsafety or potential for incorrectness than their equivalent
+items in the standard library. All `unsafe` APIs will have documentation
+explicitly detailing what the API requires you to uphold in order for it to
+function safely and correctly. All safe APIs will do so themselves.
 
 # Macro Construction
 
 Heap allocation can only occur at runtime, but the [`bitbox!`] macro will
-construct an appropriate `BitSlice` buffer at compile-time, and at run-time,
+construct an appropriate [`BitSlice`] buffer at compile-time, and at run-time,
 only copy the buffer into a heap allocation.
 
-[`BitArray`]: ../array/struct.BitArray.html
-[`BitSlice`]: ../slice/struct.BitSlice.html
-[`BitVec`]: ../vec/struct.BitVec.html
-[`bitbox!`]: ../macro.bitbox.html
+[`BitArray`]: crate::array::BitArray
+[`BitOrder`]: crate::order::BitOrder
+[`BitSlice`]: crate::slice::BitSlice
+[`BitStore`]: crate::store::BitStore
+[`BitVec`]: crate::vec::BitVec
+[`bitbox!`]: macro@crate::bitbox
 **/
 #[repr(transparent)]
 pub struct BitBox<O = Lsb0, T = usize>
@@ -130,26 +128,21 @@ where
 	O: BitOrder,
 	T: BitStore,
 {
-	pointer: NonNull<BitSlice<O, T>>,
+	bitspan: BitSpan<Mut, O, T>,
 }
 
-/// Methods specific to `BitBox<_, T>`, and not present on `Box<[T]>`.
+/// General-purpose functions not present on `Box<[T]>`.
 impl<O, T> BitBox<O, T>
 where
 	O: BitOrder,
 	T: BitStore,
 {
-	/// Clones a `&BitSlice` into a `BitVec`.
-	///
-	/// # Original
-	///
-	/// [`<Box<T: Clone> as Clone>::clone`](https://doc.rust-lang.org/alloc/boxed/struct.Box.html#impl-Clone)
+	/// Copies a [`BitSlice`] region into a new `BitBox` allocation.
 	///
 	/// # Effects
 	///
-	/// This performs a direct element-wise copy from the source slice to the
-	/// newly-allocated buffer, then sets the box to have the same starting bit
-	/// as the slice did. This allows for faster behavior.
+	/// This delegates to [`BitVec::from_bitslice`], then discards the excess
+	/// capacity.
 	///
 	/// # Examples
 	///
@@ -159,11 +152,12 @@ where
 	/// let bits = bits![0, 1, 0, 1, 1, 0, 1, 1];
 	/// let bb = BitBox::from_bitslice(&bits[2 ..]);
 	/// assert_eq!(bb, bits[2 ..]);
+	/// assert_eq!(bb.as_slice(), bits.as_slice());
 	/// ```
-	#[inline]
-	#[cfg(not(tarpaulin_include))]
+	///
+	/// [`BitVec::from_bitslice`]: crate::vec::BitVec::from_bitslice
 	pub fn from_bitslice(slice: &BitSlice<O, T>) -> Self {
-		slice.to_bitvec().into_boxed_bitslice()
+		BitVec::from_bitslice(slice).into_boxed_bitslice()
 	}
 
 	/// Converts a `Box<[T]>` into a `BitBox`<O, T>` without copying its buffer.
@@ -187,13 +181,13 @@ where
 	/// use bitvec::prelude::*;
 	///
 	/// let boxed: Box<[u8]> = Box::new([0; 4]);
+	/// let addr = boxed.as_ptr();
 	/// let bb = BitBox::<LocalBits, _>::from_boxed_slice(boxed);
 	/// assert_eq!(bb, bits![0; 32]);
+	/// assert_eq!(addr, bb.as_slice().as_ptr());
 	/// ```
 	///
-	/// [`BitSlice::MAX_ELTS`]:
-	/// ../slice/struct.BitSlice.html#associatedconstant.MAX_ELTS
-	#[inline]
+	/// [`BitSlice::MAX_ELTS`]: crate::slice::BitSlice::MAX_ELTS
 	pub fn from_boxed_slice(boxed: Box<[T]>) -> Self {
 		Self::try_from_boxed_slice(boxed)
 			.expect("Slice was too long to be converted into a `BitBox`")
@@ -222,28 +216,18 @@ where
 	/// use bitvec::prelude::*;
 	///
 	/// let boxed: Box<[u8]> = Box::new([0; 4]);
+	/// let addr = boxed.as_ptr();
 	/// let bb = BitBox::<LocalBits, _>::try_from_boxed_slice(boxed).unwrap();
 	/// assert_eq!(bb[..], bits![0; 32]);
+	/// assert_eq!(addr, bb.as_slice().as_ptr());
 	/// ```
-	#[inline]
 	pub fn try_from_boxed_slice(boxed: Box<[T]>) -> Result<Self, Box<[T]>> {
-		let len = boxed.len();
-		if len > BitSlice::<O, T>::MAX_ELTS {
-			return Err(boxed);
-		}
+		let mut boxed = ManuallyDrop::new(boxed);
 
-		let boxed = ManuallyDrop::new(boxed);
-		let base = boxed.as_ptr();
-		Ok(Self {
-			pointer: unsafe {
-				BitPtr::new_unchecked(
-					base,
-					BitIdx::ZERO,
-					len * T::Mem::BITS as usize,
-				)
-			}
-			.to_nonnull(),
-		})
+		BitPtr::from_mut_slice(&mut boxed[..])
+			.span(boxed.len() * T::Mem::BITS as usize)
+			.map(|bitspan| Self { bitspan })
+			.map_err(|_| ManuallyDrop::into_inner(boxed))
 	}
 
 	/// Converts the slice back into an ordinary slice of memory elements.
@@ -265,23 +249,90 @@ where
 	/// use bitvec::prelude::*;
 	///
 	/// let bb = bitbox![0; 5];
+	/// let addr = bb.as_slice().as_ptr();
 	/// let boxed = bb.into_boxed_slice();
 	/// assert_eq!(boxed[..], [0][..]);
+	/// assert_eq!(addr, boxed.as_ptr());
 	/// ```
-	#[inline]
 	pub fn into_boxed_slice(self) -> Box<[T]> {
-		let mut this = ManuallyDrop::new(self);
-		unsafe { Box::from_raw(this.as_mut_slice()) }
+		self.pipe(ManuallyDrop::new)
+			.as_mut_slice()
+			.pipe(|slice| unsafe { Box::from_raw(slice) })
+	}
+
+	/// Converts `self` into a vector without clones or allocation.
+	///
+	/// The resulting vector can be converted back into a box via [`BitVec<O,
+	/// T>`]’s [`.into_boxed_bitslice()`] method.
+	///
+	/// # Original
+	///
+	/// [`slice::into_vec`](https://doc.rust-lang.org/stable/std/primitive.slice.html#method.into_vec)
+	///
+	/// # API Differences
+	///
+	/// Despite taking a `Box<[T]>` receiver, this function is written in an
+	/// `impl<T> [T]` block.
+	///
+	/// Rust does not allow the text
+	///
+	/// ```rust,ignore
+	/// impl<O, T> BitSlice<O, T> {
+	///   fn into_bitvec(self: BitBox<O, T>);
+	/// }
+	/// ```
+	///
+	/// to be written, and `BitBox` exists specifically because
+	/// `Box<BitSlice<>>` cannot be written either, so this function must be
+	/// implemented directly on `BitBox` rather than on `BitSlice` with a boxed
+	/// receiver.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::prelude::*;
+	///
+	/// let bb = bitbox![0, 1, 0, 1];
+	/// let bv = bb.into_bitvec();
+	///
+	/// assert_eq!(bv, bitvec![0, 1, 0, 1]);
+	/// ```
+	///
+	/// [`BitVec<O, T>`]: crate::vec::BitVec
+	/// [`.into_boxed_bitslice()`]: crate::vec::BitVec::into_boxed_bitslice
+	pub fn into_bitvec(self) -> BitVec<O, T> {
+		let mut bitspan = self.bitspan;
+		let mut raw = self
+			//  Disarm the `self` destructor
+			.pipe(ManuallyDrop::new)
+			//  Extract the `Box<[T]>` handle, invalidating `self`
+			.with_box(|b| unsafe { ManuallyDrop::take(b) })
+			//  The distribution guarantees this to be correct and in-place.
+			.into_vec()
+			//  Disarm the `Vec<T>` destructor *also*.
+			.pipe(ManuallyDrop::new);
+		/* The distribution claims that `[T]::into_vec(Box<[T]>) -> Vec<T>` does
+		not alter the address of the heap allocation, and only modifies the
+		buffer handle. Nevertheless, update the bit-pointer with the address of
+		the vector as returned by this transformation Just In Case.
+
+		Inspection of the distribution’s implementation shows that the
+		conversion from `(buf, len)` to `(buf, cap, len)` is done by using the
+		slice length as the buffer capacity. However, this is *not* a behavior
+		guaranteed by the distribution, and so the pipeline above must remain in
+		place in the event that this behavior ever changes. It should compile
+		away to nothing, as it is almost entirely typesystem manipulation.
+		*/
+		unsafe {
+			bitspan.set_address(raw.as_mut_ptr());
+			BitVec::from_fields(bitspan, raw.capacity())
+		}
 	}
 
 	/// Views the buffer’s contents as a `BitSlice`.
 	///
 	/// This is equivalent to `&bb[..]`.
 	///
-	/// # Original
-	///
-	/// [`<Box<[T]> as AsRef<[T]>>::as_ref`](https://doc.rust-lang.org/alloc/boxed/struct.Box.html#impl-AsRef%3CT%3E)
-	///
 	/// # Examples
 	///
 	/// ```rust
@@ -290,20 +341,14 @@ where
 	/// let bb = bitbox![0, 1, 1, 0];
 	/// let bits = bb.as_bitslice();
 	/// ```
-	#[inline]
-	#[cfg(not(tarpaulin_include))]
 	pub fn as_bitslice(&self) -> &BitSlice<O, T> {
-		self.bitptr().to_bitslice_ref()
+		self.bitspan.to_bitslice_ref()
 	}
 
 	/// Extracts a mutable bit-slice of the entire vector.
 	///
 	/// Equivalent to `&mut bv[..]`.
 	///
-	/// # Original
-	///
-	/// [`<Box<[T]> as AsMut<[T]>>::as_mut`](https://doc.rust-lang.org/alloc/boxed/struct.Box.html#impl-AsMut%3CT%3E)
-	///
 	/// # Examples
 	///
 	/// ```rust
@@ -313,21 +358,15 @@ where
 	/// let bits = bv.as_mut_bitslice();
 	/// bits.set(0, true);
 	/// ```
-	#[inline]
-	#[cfg(not(tarpaulin_include))]
 	pub fn as_mut_bitslice(&mut self) -> &mut BitSlice<O, T> {
-		self.bitptr().to_bitslice_mut()
+		self.bitspan.to_bitslice_mut()
 	}
 
 	/// Extracts an element slice containing the entire box.
 	///
-	/// # Original
-	///
-	/// [`<Box<[T]> as AsRef<[T]>>::as_ref`](https://doc.rust-lang.org/alloc/boxed/struct.Box.html#impl-AsRef%3CT%3E)
-	///
 	/// # Analogue
 	///
-	/// See [`as_bitslice`] for a `&BitBox -> &BitSlice` transform.
+	/// See [`.as_bitslice()`] for a `&BitBox -> &BitSlice` transform.
 	///
 	/// # Examples
 	///
@@ -340,23 +379,19 @@ where
 	/// # }
 	/// ```
 	///
-	/// [`as_bitslice`]: #method.as_bitslice
-	#[inline]
+	/// [`.as_bitslice()`]: Self::as_bitslice
 	pub fn as_slice(&self) -> &[T] {
-		let bitptr = self.bitptr();
-		let (base, elts) = (bitptr.pointer().to_const(), bitptr.elements());
-		unsafe { slice::from_raw_parts(base, elts) }
+		let (data, len) =
+			(self.bitspan.address().to_const(), self.bitspan.elements());
+		unsafe { slice::from_raw_parts(data, len) }
 	}
 
 	/// Extracts a mutable slice of the entire box.
 	///
-	/// # Original
-	///
-	/// [`<Box<[T]> as AsMut<[T]>>::as_mut`](https://doc.rust-lang.org/alloc/boxed/struct.Box.html#impl-AsMut%3CT%3E)
-	///
 	/// # Analogue
 	///
-	/// See [`as_mut_bitslice`] for a `&mut BitBox -> &mut BitSlice` transform.
+	/// See [`.as_mut_bitslice()`] for a `&mut BitBox -> &mut BitSlice`
+	/// transform.
 	///
 	/// # Examples
 	///
@@ -369,12 +404,11 @@ where
 	/// # }
 	/// ```
 	///
-	/// [`as_mut_bitslice`]: #method.as_mut_bitslice
-	#[inline]
+	/// [`.as_mut_bitslice()`]: Self::as_mut_bitslice
 	pub fn as_mut_slice(&mut self) -> &mut [T] {
-		let bitptr = self.bitptr();
-		let (base, elts) = (bitptr.pointer().to_mut(), bitptr.elements());
-		unsafe { slice::from_raw_parts_mut(base, elts) }
+		let (data, len) =
+			(self.bitspan.address().to_mut(), self.bitspan.elements());
+		unsafe { slice::from_raw_parts_mut(data, len) }
 	}
 
 	/// Sets the uninitialized bits of the vector to a fixed value.
@@ -399,53 +433,47 @@ where
 	/// bb.set_uninitialized(true);
 	/// assert_eq!(bb.as_slice(), &[!3u8]);
 	/// ```
-	#[inline]
 	pub fn set_uninitialized(&mut self, value: bool) {
-		let head = self.bitptr().head().value() as usize;
-		let tail = head + self.len();
-		let elts = self.bitptr().elements() * T::Mem::BITS as usize;
-		let mut bp = self.bitptr();
+		let mut bp = self.bitspan;
+		let (_, head, bits) = bp.raw_parts();
+		let head = head.value() as usize;
+		let tail = head + bits;
+		let full = crate::mem::elts::<T::Mem>(tail) * T::Mem::BITS as usize;
 		unsafe {
 			bp.set_head(BitIdx::ZERO);
-			bp.set_len(elts);
-			let bits = bp.to_bitslice_mut::<O>();
+			bp.set_len(full);
+			let bits = bp.to_bitslice_mut();
 			bits.get_unchecked_mut(.. head).set_all(value);
 			bits.get_unchecked_mut(tail ..).set_all(value);
 		}
 	}
 
-	#[inline]
-	pub(crate) fn bitptr(&self) -> BitPtr<T> {
-		self.pointer.as_ptr().pipe(BitPtr::from_bitslice_ptr_mut)
-	}
-
 	/// Permits a function to modify the `Box<[T]>` backing storage of a
 	/// `BitBox<_, T>`.
 	///
-	/// This produces a temporary `Box<[T::Mem]>` structure governing the
-	/// `BitBox`’s buffer and allows a function to view it mutably. After the
+	/// This produces a temporary `Box<[T]>` structure governing the `BitBox`’s
+	/// buffer and allows a function to view it mutably. After the
 	/// callback returns, the `Box` is written back into `self` and forgotten.
 	///
 	/// # Type Parameters
 	///
-	/// - `F`: A function which operates on a mutable borrow of a
-	///   `Box<[T::Mem]>` buffer controller.
+	/// - `F`: A function which operates on a mutable borrow of a `Box<[T]>`
+	///   buffer controller.
 	/// - `R`: The return type of the `F` function.
 	///
 	/// # Parameters
 	///
 	/// - `&mut self`
-	/// - `func`: A function which receives a mutable borrow of a
-	///   `Box<[T::Mem]>` controlling `self`’s buffer.
+	/// - `func`: A function which receives a mutable borrow of a `Box<[T]>`
+	///   controlling `self`’s buffer.
 	///
 	/// # Returns
 	///
 	/// The return value of `func`. `func` is forbidden from borrowing any part
-	/// of the `Box<[T::Mem]>` temporary view.
+	/// of the `Box<[T]>` temporary view.
 	fn with_box<F, R>(&mut self, func: F) -> R
-	where F: FnOnce(&mut ManuallyDrop<Box<[T::Mem]>>) -> R {
+	where F: FnOnce(&mut ManuallyDrop<Box<[T]>>) -> R {
 		self.as_mut_slice()
-			.pipe(|s| s as *mut [T] as *mut [T::Mem])
 			.pipe(|raw| unsafe { Box::from_raw(raw) })
 			.pipe(ManuallyDrop::new)
 			.pipe_ref_mut(func)
