@@ -1,9 +1,10 @@
-//! Trait implementations for `BitBox`
+//! Non-operator trait implementations.
 
 use crate::{
 	boxed::BitBox,
+	mutability::Mut,
 	order::BitOrder,
-	pointer::BitPtr,
+	ptr::BitSpan,
 	slice::BitSlice,
 	store::BitStore,
 	vec::BitVec,
@@ -12,7 +13,6 @@ use crate::{
 use alloc::boxed::Box;
 
 use core::{
-	any,
 	borrow::{
 		Borrow,
 		BorrowMut,
@@ -74,7 +74,6 @@ where
 	}
 }
 
-#[cfg(not(tarpaulin_include))]
 impl<O, T> Eq for BitBox<O, T>
 where
 	O: BitOrder,
@@ -150,13 +149,15 @@ where
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O, T> PartialOrd<BitBox<O, T>> for BitSlice<O, T>
+impl<O1, O2, T1, T2> PartialOrd<BitBox<O2, T2>> for BitSlice<O1, T1>
 where
-	O: BitOrder,
-	T: BitStore,
+	O1: BitOrder,
+	O2: BitOrder,
+	T1: BitStore,
+	T2: BitStore,
 {
 	#[inline]
-	fn partial_cmp(&self, other: &BitBox<O, T>) -> Option<cmp::Ordering> {
+	fn partial_cmp(&self, other: &BitBox<O2, T2>) -> Option<cmp::Ordering> {
 		self.partial_cmp(other.as_bitslice())
 	}
 }
@@ -175,6 +176,34 @@ where
 }
 
 #[cfg(not(tarpaulin_include))]
+impl<'a, O1, O2, T1, T2> PartialOrd<BitBox<O2, T2>> for &'a BitSlice<O1, T1>
+where
+	O1: BitOrder,
+	O2: BitOrder,
+	T1: BitStore,
+	T2: BitStore,
+{
+	#[inline]
+	fn partial_cmp(&self, other: &BitBox<O2, T2>) -> Option<cmp::Ordering> {
+		self.partial_cmp(other.as_bitslice())
+	}
+}
+
+#[cfg(not(tarpaulin_include))]
+impl<'a, O1, O2, T1, T2> PartialOrd<BitBox<O2, T2>> for &'a mut BitSlice<O1, T1>
+where
+	O1: BitOrder,
+	O2: BitOrder,
+	T1: BitStore,
+	T2: BitStore,
+{
+	#[inline]
+	fn partial_cmp(&self, other: &BitBox<O2, T2>) -> Option<cmp::Ordering> {
+		self.partial_cmp(other.as_bitslice())
+	}
+}
+
+#[cfg(not(tarpauln_include))]
 impl<O, T> AsRef<BitSlice<O, T>> for BitBox<O, T>
 where
 	O: BitOrder,
@@ -186,7 +215,7 @@ where
 	}
 }
 
-#[cfg(not(tarpaulin_include))]
+#[cfg(not(tarpauln_include))]
 impl<O, T> AsMut<BitSlice<O, T>> for BitBox<O, T>
 where
 	O: BitOrder,
@@ -257,12 +286,11 @@ where
 	#[inline(always)]
 	fn default() -> Self {
 		Self {
-			pointer: BitPtr::EMPTY.to_nonnull(),
+			bitspan: BitSpan::<Mut, O, T>::EMPTY,
 		}
 	}
 }
 
-#[cfg(not(tarpaulin_include))]
 impl<O, T> Debug for BitBox<O, T>
 where
 	O: BitOrder,
@@ -270,16 +298,9 @@ where
 {
 	#[inline]
 	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
-		if fmt.alternate() {
-			self.bitptr().render(
-				fmt,
-				"Box",
-				Some(any::type_name::<O>()),
-				None,
-			)?;
-			fmt.write_str(" ")?;
-		}
-		Display::fmt(self.as_bitslice(), fmt)
+		Pointer::fmt(self, fmt)?;
+		fmt.write_str(" ")?;
+		Display::fmt(self, fmt)
 	}
 }
 
@@ -339,8 +360,7 @@ where
 {
 	#[inline]
 	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
-		self.bitptr()
-			.render(fmt, "Box", Some(any::type_name::<O>()), None)
+		self.as_bitspan().render(fmt, "Box", None)
 	}
 }
 
@@ -369,7 +389,23 @@ where
 	}
 }
 
+/// This is not present on `Box<[T]>`, but is needed to fit into the general
+/// operator implementations.
 #[cfg(not(tarpaulin_include))]
+impl<O, T> IntoIterator for BitBox<O, T>
+where
+	O: BitOrder,
+	T: BitStore,
+{
+	type IntoIter = <crate::vec::BitVec<O, T> as IntoIterator>::IntoIter;
+	type Item = <Self::IntoIter as Iterator>::Item;
+
+	#[inline]
+	fn into_iter(self) -> Self::IntoIter {
+		self.into_bitvec().into_iter()
+	}
+}
+
 unsafe impl<O, T> Send for BitBox<O, T>
 where
 	O: BitOrder,
@@ -377,7 +413,6 @@ where
 {
 }
 
-#[cfg(not(tarpaulin_include))]
 unsafe impl<O, T> Sync for BitBox<O, T>
 where
 	O: BitOrder,
@@ -385,7 +420,6 @@ where
 {
 }
 
-#[cfg(not(tarpaulin_include))]
 impl<O, T> Unpin for BitBox<O, T>
 where
 	O: BitOrder,
